public class Ascensor {
	private ArbolBinario abb;
	private int nPisos;
	private int piso;
	private int dir;
	
	public Ascensor (int n){
		this.nPisos = n;
		this.abb = new ArbolBinario();
		this.piso = 1;  //es ascensor parte en el 1
		this.dir = 0;   //no tiene direccion
	}
	
	public boolean presionaPiso (int x){
		
		//No se puede si pasamos el tope de pisos o 
		//si el piso no esta en el camino (direccion) del ascensor
		if(	x>this.nPisos || 
			x<1 || 
			(this.dir == 1 && x < this.piso)  ||
			(this.dir == -1 && x > this.piso) ) return false;
		
		//Si esta detenido, verificamos que sea otro piso
		if(this.dir == 0 && this.piso == x)
			return false;	
	
		//actualizamos la direccion
		this.dir = this.piso>x?-1:1;
		//Agrega el piso al ABB	
		this.abb.agregarValor(x);
		return true;
	}
	
	public int siguientePiso(){
		//Si no hay pisos
		if(this.abb.estaVacio()){
			this.dir = 0;
			return -1;
		}
		
		//Depende de la direccion
		if(this.dir>0)
			this.piso = this.abb.eliminarIzquierda();
		else if(this.dir<0)
			this.piso = this.abb.eliminarDerecha();
		
		return this.piso;	
	}
		
    
    public static void main(String[] args){
    
    	Console c = new Console();	
    	Ascensor ascensor = new Ascensor(10);
    	c.println("Ascensor en piso 1");
    	while(true){
    		c.print("Opcion: ");
    		String op = c.readLine();
    			if(op.indexOf("PISO") == 0){
    				int piso = Integer.parseInt(op.substring(5));
    				if(ascensor.presionaPiso(piso))
    					c.println("Piso "+piso+" marcado");
    				else
    					c.println("No puede ir a ese piso");
    			}
    			else if(op.indexOf("IR") == 0){
    				int piso = ascensor.siguientePiso();
    				if(piso != -1)
    					c.println("Ascensor en "+piso+" piso");
    				else
    					c.println("Ascensor detenido");
    			}		
    			else
    				c.println("Opcion Invalida");
    		System.out.println(ascensor.abb.imprime());		
    			
    	}
  	}
        
}

class Nodo{
	public int info;
	public Nodo izq, der;
	
	public Nodo(int x){
		this.info = x;
		this.izq = null;
		this.der = null;
	}
}

class ArbolBinario{
	protected Nodo raiz;
	
	public ArbolBinario(){ this.raiz = null; }
	
	String imprime(){
		String retorno = "";
		if(this.raiz!= null) {
			retorno+="raiz: "+raiz.info+"\n";
			if(this.raiz.der!= null) retorno+="raiz.der: "+raiz.der.info+"\n";
			if(this.raiz.izq!= null) retorno+="raiz.izq: "+raiz.izq.info+"\n";
		}
		else
			retorno = "raiz nula";
		return retorno;
	}
	
	public void agregarValor(int x){
		
		Nodo p = new Nodo(x);
		
		if(this.raiz==null) 
			this.raiz = p;
		else{
			//Se busca donde va el nuevo nodo
			Nodo q = this.raiz;
			Nodo f = null;
			while( q!=null){
				f=q;
				if(q.info > x)
					q = q.izq;
				else if(q.info < x)
					q = q.der;
				else  //si ya estaba
					return;
			}
			//Insercion del nuevo Nodo
			if(f.info > x)
				f.izq = p;
			else 
				f.der = p;
		}

	}
	
	public int eliminarIzquierda(){
		if(this.raiz == null) return 0;
		if(this.raiz.izq == null){
			int i = this.raiz.info;
			this.raiz = this.raiz.der;
			return i;
			
		}
		Nodo r = this.raiz;
		Nodo p = this.raiz;
		while(r.izq!= null){ 
			p = r;
			r = r.izq;
		}
		int i = r.info;
		p.izq = r.der;
		return i;
	}
	public int eliminarDerecha(){
		if(this.raiz == null) return 0;
		if(this.raiz.der == null){
			int i = this.raiz.info;
			this.raiz = this.raiz.izq;
			return i;
			
		}
		Nodo r = this.raiz;
		Nodo p = this.raiz;
		while(r.der!= null){
			p = r;
			r = r.der;
		}
		int i = r.info;
		p.der = r.izq;
		return i;
	}
			
	public boolean estaVacio(){
		return this.raiz == null;
	}
}
	